Type TWarehouse Extends THouse
	Method init_TWarehouse:TWarehouse(amount:Int, range:Int, nWorkers:Int=2, v:Double=16)
		Self.amount=amount;
		areaRange=range;
		workers=New TWorker[nWorkers]
		If(pos)
			For Local i:Int=0 To NWorkers-1
				workers[i]=New TWorker.init_TWorker(pos.copy(), v, Rand(0, 1), 5, Self)
				GameManager.addObject(workers[i]);
			Next
		EndIf
		Return Self
	End Method
	Method initArea(faction:Int)
		For Local x0:Int=coords.x-areaRange To coords.x+areaRange
			For Local y0:Int=coords.y-areaRange To coords.y+areaRange
				If(TTileMap.onTileMap(GameManager.tilemap.dimension, x0, y0))
					If(GameManager.tilemap.data[x0, y0].faction=0)
						GameManager.tilemap.data[x0, y0].faction=faction;
					EndIf
				EndIf
			Next
		Next
	End Method
	
	Method update(dt:Double)
		For Local i:Int=0 To Len(workers)-1
			If(ListIsEmpty(workers[i].actionQueue) And Not workers[i].currentAction)
				newJob(workers[i], i);
			EndIf
		Next
	End Method
	
	Method newJob(worker:TWorker, workerId:Int)
		Local path:TList=Null;
		For Local job:TJob=EachIn GameManager.players[faction].jobs
			If(job.lifetime<=9 And TTileMap.inRangeSup(job.structure.coords, coords, areaRange) And GameManager.players[faction].storage[job.id]<GameManager.players[faction].getMaxCapacity())
				path=TTileMap.AStar(Self, job.structure.tile);
				If(path)
					Local path_back:TList=path.copy();
					path_back.reverse();
					worker.addAction(New TAction_Spawn.init(worker, pos));
					worker.addAction(New TAction_WalkPath.init(worker, path));
					worker.addAction(New TAction_Rest.init(worker, 0.5));
					worker.addAction(New TAction_Take.init(worker, job.id, job.structure.goods, job.structure));
					worker.addAction(New TAction_WalkPath.init(worker, path_back));
					worker.addAction(New TAction_Put.init(worker, GameManager.player.storage, Self));
					worker.addAction(New TAction_Rest.init(worker, 5));
					worker.job=job;
					ListRemove(GameManager.players[faction].jobs, job);
					Exit
				EndIf
			EndIf
		Next
	End Method
	
	Method saveToFile(out:TStream)
		Super.saveToFile(out);
	End Method
	Method loadFromFile(in:TStream)
		Super.loadFromFile(in);
	End Method
End Type

Type TFruitPicker Extends THouse
	Field currentJobs:TFruit[];
	Method init_TFruitPicker:TFruitPicker(amount:Int, range:Int, nWorkers:Int=3, v:Double=16)
		Self.amount=amount;
		areaRange=range;
		workers=New TWorker[nWorkers];
		currentJobs=New TFruit[nWorkers];
		If(pos)
			For Local i:Int=0 To NWorkers-1
				workers[i]=New TWorker.init_TWorker(pos.copy(), v, Rand(0, 1), 5, Self)
				GameManager.addObject(workers[i]);
			Next
		EndIf
		Return Self
	End Method
	Method update(dt:Double)
		productivity=0
		For Local i:Int=0 To Len(workers)-1
			If(ListIsEmpty(workers[i].actionQueue) And Not workers[i].currentAction)
				currentJobs[i]=Null;
				newJob(workers[i], i);
				
			EndIf
			productivity:+workers[i].productivity;
		Next
		productivity:/Double(Len(workers));
		For Local i:Int=0 To Len(goods)-1
			If(goods[i]>reserved[i])
				New TJob.init(Self, 5, i)
			EndIf
		Next
			
	End Method

	Method newJob(worker:TWorker, workerId:Int)
		Local tilemap:TTileMap=GameManager.tilemap;
		Local maxAmount:Int=0;
		Local targetFruit:TFruit=Null;
		For Local x:Int=coords.x-areaRange To coords.x+areaRange
			For Local y:Int=coords.y-areaRange To coords.y+areaRange
				If(TTileMap.onTileMap(tilemap.dimension, x, y))
					If(TFruit(tilemap.data[x, y].structure) And goods[TFruit(tilemap.data[x, y].structure).fruitId]<amount And TFruit(tilemap.data[x, y].structure).amount>=1 And TFruit(tilemap.data[x, y].structure).amount>maxAmount And Not jobTaken(TFruit(tilemap.data[x, y].structure)))
						targetFruit=TFruit(tilemap.data[x, y].structure);
						maxAmount=targetFruit.amount;
					EndIf
				EndIf
			Next
		Next
		If(targetFruit)
			worker.addAction(New TAction_Spawn.init(worker, pos));
			worker.addAction(New TAction_WalkTo.init(worker, targetFruit.pos.add(New Point.init((2*Rand(0, 1)-1)*5, 1))));
			worker.addAction(New TAction_pickFruit.init(worker, 5, targetFruit));
			worker.addAction(New TAction_WalkTo.init(worker, pos));
			worker.addAction(New TAction_Put.init(worker, goods, Self));
			worker.addAction(New TAction_Rest.init(worker, 5));
			currentJobs[workerId]=targetFruit;
		EndIf
	End Method
	Method jobTaken:Int(fruit:TFruit)
		For Local fr:TFruit=EachIn currentJobs
			If(fruit=fr)
				Return True;
			EndIf
		Next
		Return False;
	End Method
	
	Method saveToFile(out:TStream)
		Super.saveToFile(out);
		WriteInt(out, Len(currentJobs))
		For Local i:Int=0 To Len(currentJobs)-1
			If(currentJobs[i])
				WriteInt(out, 1);
				WriteInt(out, currentJobs[i].coords.x);
				WriteInt(out, currentJobs[i].coords.y);
			Else
				WriteInt(out, 0);
			EndIf
		Next
	End Method
	Method loadFromFile(in:TStream)
		Super.loadFromFile(in);
		Local n:Int=ReadInt(in);
		currentJobs=New TFruit[n];
		For Local i:Int=0 To n-1
			Local exists:Int=ReadInt(in);
			If(exists)
				Local x:Int=ReadInt(in);
				Local y:Int=ReadInt(in);
				If(TFruit(GameManager.tilemap.data[x, y].structure))
					currentJobs[i]=TFruit(GameManager.tilemap.data[x, y].structure);
				EndIf
			EndIf
		Next
	End Method
End Type

Type TCakeBaker Extends THouse
	Field rate:Int;
	Field timer:Double, cooldown:Double;
	Field work_timer:Double, total_timer:Double;

	Method init_TCakeBaker:TCakeBaker(amount:Int, range:Int, rate:Int, nWorkers:Int=3, v:Double=16)
		Self.amount=amount;
		areaRange=range;
		workers=New TWorker[nWorkers];
		Self.rate=rate;
		If(rate>0)
			cooldown=60/rate;
		EndIf
		timer=cooldown;
		If(pos)
			For Local i:Int=0 To NWorkers-1
				workers[i]=New TWorker.init_TWorker(pos.copy(), v, Rand(0, 1), 5, Self)
				GameManager.addObject(workers[i]);
			Next
		EndIf
		Return Self
	End Method
	Method update(dt:Double)
		total_timer:+dt;
		If(goods[TGoods.GOODS_APPLES]>=1 And goods[TGoods.GOODS_FLOUR]>=1)
			work_timer:+dt;
			timer:-dt;
			If(timer<=0)
				timer=cooldown;
				goods[TGoods.GOODS_APPLES]:-1;
				goods[TGoods.GOODS_FLOUR]:-1;
				goods[TGoods.GOODS_CAKE]:+1;
			EndIf
		EndIf
		For Local i:Int=0 To Len(workers)-1
			If(ListIsEmpty(workers[i].actionQueue) And Not workers[i].currentAction)
				If( goods[TGoods.GOODS_APPLES]=0)
					newJob(workers[i], i, TGoods.GOODS_APPLES);
				ElseIf(goods[TGoods.GOODS_FLOUR]=0)
					newJob(workers[i], i, TGoods.GOODS_FLOUR);
				ElseIf(goods[TGoods.GOODS_APPLES]<amount And goods[TGoods.GOODS_FLOUR]<amount And Rand(0, 1))
					newJob(workers[i], i, TGoods.GOODS_APPLES);
				ElseIf(goods[TGoods.GOODS_FLOUR]<amount)
					newJob(workers[i], i, TGoods.GOODS_FLOUR);
				EndIf
			EndIf
		Next
		If(goods[TGoods.GOODS_CAKE]>reserved[TGoods.GOODS_CAKE])
			New TJob.init(Self, 5, TGoods.GOODS_CAKE)
		EndIf
		If(total_timer>=5)
			productivity=(0.8*productivity+0.2*work_timer/total_timer);
			work_timer=0;
			total_timer=0;
		EndIf
	End Method
	Method newJob(worker:TWorker, workerId:Int, ingredientsId:Int)
		Local tilemap:TTileMap=GameManager.tilemap;
		Local maxAmount:Int=0;
		Local maxBuilding:THouse=Null;
		Local path:TList=Null;
		For Local job:TJob=EachIn GameManager.players[faction].jobs
			If(job.id=ingredientsId)
				path=TTileMap.AStar(Self, job.structure.tile);
				If(path)
					Local path_back:TList=path.copy();
					path_back.reverse();
					worker.addAction(New TAction_Spawn.init(worker, pos));
					worker.addAction(New TAction_WalkPath.init(worker, path));
					worker.addAction(New TAction_Rest.init(worker, 0.5));
					worker.addAction(New TAction_Take.init(worker, ingredientsId, job.structure.goods, job.structure));
					worker.addAction(New TAction_WalkPath.init(worker, path_back));
					worker.addAction(New TAction_Put.init(worker, goods, Self));
					worker.addAction(New TAction_Rest.init(worker, 5));
					worker.job=job;
					ListRemove(GameManager.players[faction].jobs, job);
					Exit
				EndIf
			EndIf
		Next
		takeFromWareHouse(worker, ingredientsId);

	End Method
	Method jobTaken:Int(fruit:TFruit)

		Return False;
	End Method
	
	Method saveToFile(out:TStream)
		Super.saveToFile(out);
		WriteInt(out, rate);
		WriteDouble(out, timer);
		WriteDouble(out, cooldown);
		WriteDouble(out, work_timer);
		WriteDouble(out, total_timer);
	End Method
	Method loadFromFile(in:TStream)
		Super.loadFromFile(in);
		rate=ReadInt(in);
		timer=ReadDouble(in);
		cooldown=ReadDouble(in);
		work_timer=ReadDouble(in);
		total_timer=ReadDouble(in);
	End Method
	
End Type

Type TFarm Extends THouse
	Field currentJobs:TCrop[];
	Method init_TFarm:TFarm(amount:Int, range:Int, nWorkers:Int=3, v:Double=16)
		Self.amount=amount;
		areaRange=range;
		workers=New TWorker[nWorkers];
		currentJobs=New TCrop[nWorkers];
		If(pos)
			For Local i:Int=0 To NWorkers-1
				workers[i]=New TWorker.init_TWorker(pos.copy(), v, Rand(0, 1), 5, Self)
				GameManager.addObject(workers[i]);
			Next
		EndIf
		Return Self
	End Method
	Method update(dt:Double)
		productivity=0;
		For Local i:Int=0 To Len(workers)-1
			If(ListIsEmpty(workers[i].actionQueue) And Not workers[i].currentAction And goods[TGoods.GOODS_CROP]<amount)
				currentJobs[i]=Null;
				newJob(workers[i], i);
			EndIf
			productivity:+workers[i].productivity;
		Next
		productivity:/Double(Len(workers));
		If(goods[TGoods.GOODS_CROP]>reserved[TGoods.GOODS_CROP])
			New TJob.init(Self, 5, TGoods.GOODS_CROP)
		EndIf
	End Method

	Method newJob(worker:TWorker, workerId:Int)
		Local tilemap:TTileMap=GameManager.tilemap;
		Local maxAmount:Int=0;
		Local targetCrop:TCrop=Null;
		For Local x:Int=coords.x-areaRange To coords.x+areaRange
			For Local y:Int=coords.y-areaRange To coords.y+areaRange
				If(TTileMap.onTileMap(tilemap.dimension, x, y))
					If(TCrop(tilemap.data[x, y].structure) And TCrop(tilemap.data[x, y].structure).amount>=1 And TCrop(tilemap.data[x, y].structure).amount>maxAmount And Not jobTaken(TCrop(tilemap.data[x, y].structure)))
						targetCrop=TCrop(tilemap.data[x, y].structure);
						maxAmount=targetCrop.amount;
					EndIf
				EndIf
			Next
		Next
		If(targetCrop)
			worker.addAction(New TAction_Spawn.init(worker, pos));
			worker.addAction(New TAction_WalkTo.init(worker, targetCrop.pos.add(New Point.init((2*Rand(0, 1)-1)*5, 9))));
			worker.addAction(New TAction_pickCrop.init(worker, 5, targetCrop));
			worker.addAction(New TAction_WalkTo.init(worker, pos));
			worker.addAction(New TAction_Put.init(worker, goods, Self));
			worker.addAction(New TAction_Rest.init(worker, 5));
			currentJobs[workerId]=targetCrop;
		EndIf
	End Method
	Method jobTaken:Int(crop:TCrop)
		For Local cr:TCrop=EachIn currentJobs
			If(crop=cr)
				Return True;
			EndIf
		Next
		Return False;
	End Method
	
	Method saveToFile(out:TStream)
		Super.saveToFile(out);
		WriteInt(out, Len(currentJobs))
		For Local i:Int=0 To Len(currentJobs)-1
			If(currentJobs[i])
				WriteInt(out, 1);
				WriteInt(out, currentJobs[i].coords.x);
				WriteInt(out, currentJobs[i].coords.y);
			Else
				WriteInt(out, 0);
			EndIf
		Next
	End Method
	Method loadFromFile(in:TStream)
		Super.loadFromFile(in);
		Local n:Int=ReadInt(in);
		currentJobs=New TCrop[n];
		For Local i:Int=0 To n-1
			Local exists:Int=ReadInt(in);
			If(exists)
				Local x:Int=ReadInt(in);
				Local y:Int=ReadInt(in);
				If(TCrop(GameManager.tilemap.data[x, y].structure))
					currentJobs[i]=TCrop(GameManager.tilemap.data[x, y].structure);
				EndIf
			EndIf
		Next
	End Method
	
End Type

Type TWindMill Extends THouse
	Field rate:Int;
	Field timer:Double, cooldown:Double;
	Field animFrame:Int, animTimer:Double, animDuration:Double;
	Field work_timer:Double, total_timer:Double;

	Method init_TWindMill:TWindMill(amount:Int, range:Int, rate:Int, nWorkers:Int=1, v:Double=16)
		Self.amount=amount;
		areaRange=range;
		workers=New TWorker[nWorkers];
		Self.rate=rate;
		If(rate>0)
			cooldown=60/rate;
		EndIf
		timer=cooldown;
		animDuration=0.1;
		animTimer=animDuration;
		If(pos)
			For Local i:Int=0 To NWorkers-1
				workers[i]=New TWorker.init_TWorker(pos.copy(), v, Rand(0, 1), 5, Self)
				GameManager.addObject(workers[i]);
			Next
		EndIf
		Return Self
	End Method
	Method update(dt:Double)
		total_timer:+dt;
		If(goods[TGoods.GOODS_CROP]>=1)
			work_timer:+dt;
			timer:-dt;
			If(timer<=0)
				timer=cooldown;
				goods[TGoods.GOODS_CROP]:-1;
				goods[TGoods.GOODS_FLOUR]:+1;
			EndIf
			animTimer:-dt;
			If(animTimer<=0)
				animFrame=1-animFrame;
				animTimer=animDuration;
			EndIf
		Else
			If(animFrame=1)
				animTimer:-dt;
				If(animTimer<=0)
					animFrame=0;
				EndIf
				animTimer=animDuration;
			EndIf
		EndIf
		For Local i:Int=0 To Len(workers)-1
			If(ListIsEmpty(workers[i].actionQueue) And Not workers[i].currentAction)
				If( goods[TGoods.GOODS_CROP]<amount)
					newJob(workers[i], i, TGoods.GOODS_CROP);
				EndIf
			EndIf
		Next
		If(goods[TGoods.GOODS_FLOUR]>reserved[TGoods.GOODS_FLOUR])
			New TJob.init(Self, 5, TGoods.GOODS_FLOUR)
		EndIf
		If(total_timer>=5)
			productivity=(0.8*productivity+0.2*work_timer/total_timer);
			work_timer=0;
			total_timer=0;
		EndIf
	End Method
	Method newJob(worker:TWorker, workerId:Int, ingredientsId:Int)
		Local tilemap:TTileMap=GameManager.tilemap;
		Local maxAmount:Int=0;
		Local maxBuilding:THouse=Null;
		Local path:TList=Null;
		For Local job:TJob=EachIn GameManager.players[faction].jobs
			If(job.id=ingredientsId)
				path=TTileMap.AStar(Self, job.structure.tile);
				If(path)
					Local path_back:TList=path.copy();
					path_back.reverse();
					worker.addAction(New TAction_Spawn.init(worker, pos));
					worker.addAction(New TAction_WalkPath.init(worker, path));
					worker.addAction(New TAction_Rest.init(worker, 0.5));
					worker.addAction(New TAction_Take.init(worker, ingredientsId, job.structure.goods, job.structure));
					worker.addAction(New TAction_WalkPath.init(worker, path_back));
					worker.addAction(New TAction_Put.init(worker, goods, Self));
					worker.addAction(New TAction_Rest.init(worker, 5));
					worker.job=job;
					ListRemove(GameManager.players[faction].jobs, job);
					Exit
				EndIf
			EndIf
		Next
		takeFromWareHouse(worker, ingredientsId);
	End Method
	Method draw(scroll:Point)
		DrawImage(TResManager.img[imgId], Floor(pos.x+scroll.x), Floor(pos.y+scroll.y), frame+animFrame);
	End Method
	
	Method saveToFile(out:TStream)
		Super.saveToFile(out);
		WriteInt(out, rate);
		WriteDouble(out, timer);
		WriteDouble(out, cooldown);
		WriteInt(out, animFrame);
		WriteDouble(out, animTimer);
		WriteDouble(out, animDuration);
		WriteDouble(out, work_timer);
		WriteDouble(out, total_timer);
			
	End Method
	Method loadFromFile(in:TStream)
		Super.loadFromFile(in);
		rate=ReadInt(in);
		timer=ReadDouble(in);
		cooldown=ReadDouble(in);
		animFrame=ReadInt(in);
		animTimer=ReadDouble(in);
		animDuration=ReadDouble(in);
		work_timer=ReadDouble(in);
		total_timer=ReadDouble(in);
	End Method
End Type

Type TVendor Extends THouse
	Method init_TVendor:TVendor()
		Return Self
	End Method
	Method update(dt:Double)
		
	End Method
	
	Method saveToFile(out:TStream)
		Super.saveToFile(out);
	End Method
	Method loadFromFile(in:TStream)
		Super.loadFromFile(in);
	End Method
End Type

Type TJuiceFactory Extends THouse
	Field rate:Int;
	Field timer:Double, cooldown:Double;
	Field work_timer:Double, total_timer:Double;

	Method init_TJuiceFactory:TJuiceFactory(amount:Int, range:Int, rate:Int, nWorkers:Int=1, v:Double=16)
		Self.amount=amount;
		areaRange=range;
		workers=New TWorker[nWorkers];
		Self.rate=rate;
		If(rate>0)
			cooldown=60/rate;
		EndIf
		timer=cooldown;
		If(pos)
			For Local i:Int=0 To NWorkers-1
				workers[i]=New TWorker.init_TWorker(pos.copy(), v, Rand(0, 1), 5, Self)
				GameManager.addObject(workers[i]);
			Next
		EndIf
		Return Self
	End Method
	Method update(dt:Double)
		total_timer:+dt;
		If(goods[TGoods.GOODS_BERRIES]>=1 And goods[TGoods.GOODS_CHERRIES]>=1)
			work_timer:+dt;
			timer:-dt;
			If(timer<=0)
				timer=cooldown;
				goods[TGoods.GOODS_BERRIES]:-1;
				goods[TGoods.GOODS_CHERRIES]:-1;
				goods[TGoods.GOODS_JUICE]:+1;
			EndIf
		EndIf
		For Local i:Int=0 To Len(workers)-1
			If(ListIsEmpty(workers[i].actionQueue) And Not workers[i].currentAction)
				If( goods[TGoods.GOODS_BERRIES]=0)
					newJob(workers[i], i, TGoods.GOODS_BERRIES);
				ElseIf(goods[TGoods.GOODS_CHERRIES]=0)
					newJob(workers[i], i, TGoods.GOODS_CHERRIES);
				ElseIf(goods[TGoods.GOODS_BERRIES]<amount And goods[TGoods.GOODS_CHERRIES]<amount And Rand(0, 1))
					newJob(workers[i], i, TGoods.GOODS_BERRIES);
				ElseIf(goods[TGoods.GOODS_CHERRIES]<amount)
					newJob(workers[i], i, TGoods.GOODS_CHERRIES);
				EndIf
			EndIf
		Next
		If(goods[TGoods.GOODS_JUICE]>reserved[TGoods.GOODS_JUICE])
			New TJob.init(Self, 5, TGoods.GOODS_JUICE)
		EndIf
		If(total_timer>=5)
			productivity=(0.8*productivity+0.2*work_timer/total_timer);
			work_timer=0;
			total_timer=0;
		EndIf
	End Method
	Method newJob(worker:TWorker, workerId:Int, ingredientsId:Int)
		Local tilemap:TTileMap=GameManager.tilemap;
		Local maxAmount:Int=0;
		Local maxBuilding:THouse=Null;
		Local path:TList=Null;
		For Local job:TJob=EachIn GameManager.players[faction].jobs
			If(job.id=ingredientsId)
				path=TTileMap.AStar(Self, job.structure.tile);
				If(path)
					Local path_back:TList=path.copy();
					path_back.reverse();
					worker.addAction(New TAction_Spawn.init(worker, pos));
					worker.addAction(New TAction_WalkPath.init(worker, path));
					worker.addAction(New TAction_Rest.init(worker, 0.5));
					worker.addAction(New TAction_Take.init(worker, ingredientsId, job.structure.goods, job.structure));
					worker.addAction(New TAction_WalkPath.init(worker, path_back));
					worker.addAction(New TAction_Put.init(worker, goods, Self));
					worker.addAction(New TAction_Rest.init(worker, 5));
					worker.job=job;
					ListRemove(GameManager.players[faction].jobs, job);
					Exit
				EndIf
			EndIf
		Next
		takeFromWareHouse(worker, ingredientsId);

	End Method
	Method jobTaken:Int(fruit:TFruit)

		Return False;
	End Method
	
	Method saveToFile(out:TStream)
		Super.saveToFile(out);
		WriteInt(out, rate);
		WriteDouble(out, timer);
		WriteDouble(out, cooldown);
		WriteDouble(out, work_timer);
		WriteDouble(out, total_timer);
	End Method
	Method loadFromFile(in:TStream)
		Super.loadFromFile(in);
		rate=ReadInt(in);
		timer=ReadDouble(in);
		cooldown=ReadDouble(in);
		work_timer=ReadDouble(in);
		total_timer=ReadDouble(in);
	End Method
End Type