Type TGameManager
	Const STATE_START:Int=0;
	Const STATE_GAME:Int=1;
	Field tilemap:TTileMap;
	Field world:TWorld;
	Field players:TControl[];
	Field player:TPlayer;
	Field objectList:TList;
	Field drawList:TList;
	Field testList:TList;
	Field state:Int;
	Field scroll:Point;
	Field mainmenue:TMainMenue;
	Field speed:Int;
	Method init:TGameManager()
		state =STATE_START;
		speed=1;
		mainmenue=New TMainMenue.init();
		Return Self;
	End Method
	
	Method newGame()
		tilemap=New TTileMap.init(60, 60);
		objectList=New TList; drawList=New TList; testList=New TList;
		scroll=New Point.init(0, 0);
		
		TControl.loadBuildTemplates();
		player=New TPlayer.init(tilemap, 1000);
		player.faction=2;
		players=New TControl[3];
		players[player.faction]=player;
		players[1]=New TTown.init(1000);
		players[1].faction=1;
		For Local i:Int=0 To 2
			If(i<>player.faction And Not players[i])
				players[i]=New TControl.init_control(1000);
			EndIf
		Next
		world=New TWorld.init(tilemap, New Point.init(30, 30), 7);
		world.createWorld();
		Local startingPoints:Point[]=[New Point.init(29, 10), New Point.init(29, 49), New Point.init(10, 29), New Point.init(49, 29)];
		Local startingVendors:Point[]=[New Point.init(29, 21), New Point.init(34, 41), New Point.init(21, 29), New Point.init(41, 34)];
		Local r:Int=Rand(0, 3);
		Local startingPoint:Point=startingPoints[r];
		world.placeStartingPoint(player, tilemap.data[startingPoint.x, startingPoint.y]);
		world.placeVendor(player, tilemap.data[startingVendors[r].x, startingVendors[r].y]);

		setScroll(TTileMap.getPos(startingPoint.x, startingPoint.y));
	End Method
	
	Method endGame()
		
		world=Null;
		players=Null;
		For Local obj:TGameObject=EachIn objectList
			obj.sortCoords=Null
			If(TStructure(obj))
				TStructure(obj).tile=Null;
				TStructure(obj).coords=Null;
			EndIf
			
			If(THouse(obj))
				THouse(obj).workers=Null
			EndIf
			If(TWorker(obj))
				ClearList(TWorker(obj).actionqueue);
				TWorker(obj).currentAction=Null;
				TWorker(obj).job=Null;
				TWorker(obj).house=Null;
			EndIf
		Next
		For Local obj:TGameObject=EachIn drawList
			If(TTree(obj))
				TTree(obj).structure=Null;
			EndIf
		Next
		For Local x:Int=0 To tilemap.dimension.x-1
			For Local y:Int=0 To tilemap.dimension.y-1
				tilemap.data[x, y].structure=Null;
			Next
		Next
		tilemap.data=Null;
		tilemap=Null;
		ClearList(objectList);
		ClearList(drawList);
		ClearList(testList);
		player=Null;
		speed=1;
		state=STATE_START;
	End Method
	
	Method setScroll(p:Point)
		scroll.init(400-p.x, 300-p.y);
	End Method
	
	Method update(dt:Double)
		Select(state)
			Case STATE_START
				
				'newGame();
				'loadGame();
				'state=STATE_GAME;
				mainmenue.update();
				If(KeyHitEsc)
					End
				EndIf
			Case STATE_GAME
				'Print CountList(drawList)+", "+CountList(objectList);
				scrollUpdate(2*dt/GameManager.speed);
				For Local i:Int=0 To 2
					If(players[i])
						players[i].update(dt);
					EndIf
				Next
				For Local obj:TGameObject = EachIn objectList
					If(obj.active)
						If(Not obj.noUpdate)
							obj.update(dt);
						EndIf
					Else
						If(obj._link)
							RemoveLink(obj._link);
						EndIf
						If(obj._link_buildings)
							RemoveLink(obj._link_buildings)
						EndIf
						If(obj._link_draw)
							RemoveLink(obj._link_draw)
						EndIf
					EndIf
				Next
				If(KeyHitEsc)
					autosave();
					endGame();
					state=STATE_START;
				EndIf
		End Select
		
	End Method
	
	Method draw()
		Select(state)
			Case STATE_START
				mainmenue.draw();
			Case STATE_GAME
				tilemap.draw(scroll);
				'wip
				ClearList(testlist);
				For Local obj:TGameObject=EachIn drawlist
					If(obj.onScreen(scroll))
						ListAddFirst(testlist, obj);
					EndIf
				Next
				SortList(testlist, True, TGameObject.objCompare);
				For Local obj:TGameObject = EachIn testList
					If(obj.active)
						obj.setDraw(scroll);
						obj.draw(scroll);
					EndIf
				Next
				SetColor(255, 255, 255); SetBlend(MASKBLEND); SetAlpha(1);
				
				player.drawUI(scroll);
		End Select

	End Method
	
	Method addObject(obj:TGameObject)
		If(obj)
			obj.active=True;
			'obj._link=	ListAddFirst(objectList, obj);

			If(TFruit(obj))
				obj._link=ListAddFirst(objectList, obj);
				For Local tree:TTree=EachIn TFruit(obj).fruits
					tree.active=True;
					tree._link_draw=ListAddFirst(drawList, tree);
				Next
			ElseIf(TForest(obj))
				'obj._link=ListAddFirst(objectList, obj);
				For Local tree:TTree=EachIn TForest(obj).trees
					tree.active=True;
					tree._link_draw=ListAddFirst(drawList, tree);
				Next
			ElseIf(THouse(obj) Or TWorker(obj) Or TCrop(obj) Or(TStreet(obj)))
				obj._link=ListAddFirst(objectList, obj);
				obj._link_draw=ListAddFirst(drawList, obj);
			Else
				obj._link=ListAddFirst(objectList, obj);
			EndIf
		EndIf
	End Method 
	Method scrollUpdate(dt:Double)
		Local scrollV:Double=256;
		If(KeyDown(KEY_LEFT))
			scroll.x:+dt*scrollV
		EndIf
		If(KeyDown(KEY_RIGHT))
			scroll.x:-dt*scrollV
		EndIf
		If(KeyDown(KEY_UP))
			scroll.y:+dt*scrollV
		EndIf
		If(KeyDown(KEY_DOWN))
			scroll.y:-dt*scrollV
		EndIf
		scroll.x=Int(scroll.x);
		scroll.y=Int(scroll.y);
		If(scroll.x>tilemap.dimension.y*TTileMap.TILESIZE/2+TTileMap.TILESIZE)
			scroll.x=tilemap.dimension.y*TTileMap.TILESIZE/2+TTileMap.TILESIZE;
		EndIf
		If(scroll.x<-tilemap.dimension.x*TTileMap.TILESIZE/2+800-TTileMap.TILESIZE-112)
			scroll.x=-tilemap.dimension.x*TTileMap.TILESIZE/2+800-TTileMap.TILESIZE-112;
		EndIf
		If(scroll.y>TTileMap.TILESIZE/2+24)
			scroll.y=TTileMap.TILESIZE/2+24;
		EndIf
		If(scroll.y<-(tilemap.dimension.y+tilemap.dimension.x)*TTileMap.TILESIZE/4+600-TTileMap.TILESIZE/2)
			scroll.y=-(tilemap.dimension.y+tilemap.dimension.x)*TTileMap.TILESIZE/4+600-TTileMap.TILESIZE/2;
		EndIf
	End Method
	
	Method saveGameToFile(filename:String)
		Local out:TStream=WriteFile(filename);
		Local count:Int[]=New Int[14];
		For Local obj:TGameObject=EachIn objectList
			If(Not obj.active) Continue;
			If(TFruit(obj))
				count[2]:+1;
			ElseIf(TCrop(obj))
				count[3]:+1;
			ElseIf(TWareHouse(obj))
				count[4]:+1;
			ElseIf(TCakeBaker(obj))
				count[5]:+1;
			ElseIf(TFruitPicker(obj))
				count[6]:+1;
			ElseIf(TVendor(obj))
				count[7]:+1;
			ElseIf(TJuiceFactory(obj))
				count[8]:+1;
			ElseIf(TFarm(obj))
				count[9]:+1;
			ElseIf(TWindMill(obj))
				count[10]:+1;
			ElseIf(TTownHouse(obj))
				count[11]:+1;
			ElseIf(TMonument(obj))
				count[12]:+1;
			ElseIf(TWorker(obj))
				If(Not TWorker(obj).house) Continue;
				count[13]:+1;
			EndIf
		Next
		For Local x:Int=0 To tilemap.dimension.x-1
			For Local y:Int=0 To tilemap.dimension.y-1
				If(TForest(tilemap.data[x, y].structure))
					count[1]:+1;
				EndIf
			Next
		Next
		'tilemap
		WriteInt(out, tilemap.dimension.x);
		WriteInt(out, tilemap.dimension.y);
		WriteInt(out, Int(scroll.x));
		WriteInt(out, Int(scroll.y));
		For Local x:Int=0 To tilemap.dimension.x-1
			For Local y:Int=0 To tilemap.dimension.y-1
				tilemap.data[x, y].saveToFile(out)
				If(TStreet(tilemap.data[x, y].structure))
					WriteInt(out, 1);
					TStreet(tilemap.data[x, y].structure).saveToFile(out);
				Else
					WriteInt(out, 0);
				EndIf
			Next
		Next
		'player
		player.saveToFile(out);
		'forest
		WriteInt(out, count[1]);
		For Local x:Int=0 To tilemap.dimension.x-1
			For Local y:Int=0 To tilemap.dimension.y-1
				If(TForest(tilemap.data[x, y].structure))
					TForest(tilemap.data[x, y].structure).saveToFile(out);
				EndIf
			Next
		Next
		'fruits
		WriteInt(out, count[2]);
		For Local fruit:TFruit=EachIn objectList
			If(Not fruit.active) Continue;
			fruit.saveToFile(out);
		Next
		'crop
		WriteInt(out, count[3]);
		For Local crop:TCrop=EachIn objectList
			If(Not crop.active) Continue;
			crop.saveToFile(out);
		Next
		'warehouse
		WriteInt(out, count[4]);
		For Local warehouse:TWareHouse=EachIn objectList
			If(Not warehouse.active) Continue;
			warehouse.saveToFile(out);
		Next
		'cakebaker
		WriteInt(out, count[5]);
		For Local cakebaker:TCakeBaker=EachIn objectList
			If(Not cakebaker.active) Continue;
			cakebaker.saveToFile(out);
		Next
		'fruitpicker
		WriteInt(out, count[6]);
		For Local fruitpicker:TFruitpicker=EachIn objectList
			If(Not fruitpicker.active) Continue;
			fruitpicker.saveToFile(out);
		Next
		'vendor
		WriteInt(out, count[7]);
		For Local vendor:TVendor=EachIn objectList
			If(Not vendor.active) Continue;
			vendor.saveToFile(out);
		Next
		'juicefactory
		WriteInt(out, count[8]);
		For Local juicefactory:TJuiceFactory=EachIn objectList
			If(Not juicefactory.active) Continue;
			juicefactory.saveToFile(out);
		Next
		'farm
		WriteInt(out, count[9]);
		For Local farm:TFarm=EachIn objectList
			If(Not farm.active) Continue;
			farm.saveToFile(out);
		Next
		'windmill
		WriteInt(out, count[10]);
		For Local windmill:TWindMill=EachIn objectList
			If(Not windmill.active) Continue;
			windmill.saveToFile(out);
		Next
		'townhouse
		WriteInt(out, count[11]);
		For Local townhouse:TTownHouse=EachIn objectList
			If(Not townhouse.active) Continue;
			townhouse.saveToFile(out);
		Next
		'monument
		WriteInt(out, count[12]);
		For Local monument:TMonument=EachIn objectList
			If(Not monument.active) Continue;
			monument.saveToFile(out);
		Next
		'worker
		WriteInt(out, count[13]);
		For Local worker:TWorker=EachIn objectList
			If(Not worker.active Or Not worker.house) Continue;
			worker.saveToFile(out);
		Next
		
		'joblists
		For Local control:TControl=EachIn players
			WriteInt(out, CountList(control.jobs));
			For Local job:TJob=EachIn control.jobs
				job.saveToFile(out);
			Next
		Next
	End Method
	
	Method loadGameFromFile(filename:String)
		Local in:TStream=ReadFile(filename)
		If(Not in) Return;
		
		'tilemap
		Local dimx:Int=ReadInt(in);
		Local dimy:Int=ReadInt(in);
		tilemap=New TTileMap.init(dimx, dimy);
		objectList=New TList; drawList=New TList; testList=New TList;
		Local scrollx:Int=ReadInt(in);
		Local scrolly:Int=ReadInt(in);
		scroll=New Point.init(scrollx, scrolly);
		For Local x:Int=0 To dimx-1
			For Local y:Int=0 To dimy-1
				tilemap.data[x, y].loadFromFile(in);
				Local buildStreet:Int=ReadInt(in);
				If(buildStreet)
					Local street:TStreet=New TStreet;
					street.loadFromFile(in);
					tilemap.data[x, y].placeStructure(New TStreet.init_TStreet(tilemap.data[x, y], street.price, street.name));
					TStreet.updateId3x3(tilemap, x, y);
				EndIf
			Next
		Next

		'player
		If(TControl.buildTemplates=Null)
			TControl.loadBuildTemplates();
		EndIf
		Local p:TPlayer=New TPlayer;
		player=New TPlayer.init(tilemap, 0);
		player.loadFromFile(in);
		players=New TControl[3];
		players[player.faction]=player;
		players[1]=New TTown.init(1000);
		players[1].faction=1;
		For Local i:Int=0 To 2
			If(i<>player.faction And Not players[i])
				players[i]=New TControl.init_control(1000);
			EndIf
		Next
			
		world=New TWorld.init(tilemap, New Point.init(30, 30), 7);

		'forest
		Local n:Int=ReadInt(in);
		For Local i:Int=0 To n-1
			Local forest:TForest=New TForest
			forest.loadFromFile(in);
			forest.tile.placeStructure(forest);
		Next

		'fruits
		n=ReadInt(in);
		For Local i:Int=0 To n-1
			Local fruit:TFruit=New TFruit
			fruit.loadFromFile(in);
			fruit.tile.placeStructure(fruit);
		Next

		'crop
		n=ReadInt(in);
		For Local i:Int=0 To n-1
			Local crop:TCrop=New TCrop
			crop.loadFromFile(in);
			crop.tile.placeStructure(crop);
		Next
		
		'warehouse
		n=ReadInt(in);
		For Local i:Int=0 To n-1
			Local warehouse:TWareHouse=New TWareHouse
			warehouse.loadFromFile(in);
			warehouse.tile.placeStructure(warehouse);
		Next
		
		'cakebaker
		n=ReadInt(in);
		For Local i:Int=0 To n-1
			Local cakebaker:TCakeBaker=New TCakeBaker
			cakebaker.loadFromFile(in);
			cakebaker.tile.placeStructure(cakebaker);
		Next
		
		'fruitpicker
		n=ReadInt(in);
		For Local i:Int=0 To n-1
			Local fruitpicker:TFruitPicker=New TFruitPicker
			fruitpicker.loadFromFile(in);
			fruitpicker.tile.placeStructure(fruitpicker);
		Next
		
		'vendor
		n=ReadInt(in);
		For Local i:Int=0 To n-1
			Local vendor:TVendor=New TVendor
			vendor.loadFromFile(in);
			vendor.tile.placeStructure(vendor);
		Next
		
		'juicefactory
		n=ReadInt(in);
		For Local i:Int=0 To n-1
			Local juicefactory:TJuiceFactory=New TJuiceFactory
			juicefactory.loadFromFile(in);
			juicefactory.tile.placeStructure(juicefactory);
		Next
		
		'farm
		n=ReadInt(in);
		For Local i:Int=0 To n-1
			Local farm:TFarm=New TFarm
			farm.loadFromFile(in);
			farm.tile.placeStructure(farm);
		Next
		
		'windmill
		n=ReadInt(in);
		For Local i:Int=0 To n-1
			Local windmill:TWindMill=New TWindMill
			windmill.loadFromFile(in);
			windmill.tile.placeStructure(windmill);
		Next
		
		'townhouse
		n=ReadInt(in);
		For Local i:Int=0 To n-1
			Local townhouse:TTownHouse=New TTownHouse
			townhouse.loadFromFile(in);
			townhouse.tile.placeStructure(townhouse);
		Next
		
		'monument
		n=ReadInt(in);
		For Local i:Int=0 To n-1
			Local monument:TMonument=New TMonument
			monument.loadFromFile(in);
			monument.tile.placeStructure(monument);
			TTown(players[1]).monument=monument;
		Next
		'worker
		n=ReadInt(in);
		For Local i:Int=0 To n-1
			Local worker:TWorker=New TWorker
			worker.loadFromFile(in);
			If(worker.house)
				GameManager.addObject(worker);
			EndIf
		Next
		'joblists
		For Local i:Int=0 To 2
			Local n:Int=ReadInt(in);
			For Local j:Int=0 To n-1
				Local job:TJob=New TJob;
				job.loadFromFile(in);
				If(job.structure)
					ListAddLast(players[i].jobs, job);
				EndIf
			Next
		Next
	End Method
End Type

Type TMainMenue
	Field button_newgame:TButton_Text;
	Field button_quicksave:TButton_Text;
	Field button_autosave:TButton_Text;
	Field button_endgame:TButton_Text;

	Method init:TMainMenue()
		button_newgame=New TButton_Text.init_text(New Point.init(300, 100), 200, 40, 0, TResManager.IMG_BUTTONS3, 0, "New Game")
		button_quicksave=New TButton_Text.init_text(New Point.init(300, 150), 200, 40, 0, TResManager.IMG_BUTTONS3, 0, "Load Quicksave")
		button_autosave=New TButton_Text.init_text(New Point.init(300, 200), 200, 40, 0, TResManager.IMG_BUTTONS3, 0, "Load Autosave")
		button_endgame=New TButton_Text.init_text(New Point.init(300, 250), 200, 40, 0, TResManager.IMG_BUTTONS3, 0, "End Game")
		Return Self;
	End Method
	
	Method update()
		If(button_newgame.hit())
			GameManager.newGame();
			GameManager.state=TGameManager.STATE_GAME
		ElseIf(button_quicksave.hit() And FileType("quicksave.sav")=1)
			GameManager.loadGameFromFile("quicksave.sav");
			GameManager.state=TGameManager.STATE_GAME
		ElseIf(button_autosave.hit() And FileType("autosave.sav")=1)
			GameManager.loadGameFromFile("autosave.sav");
			GameManager.state=TGameManager.STATE_GAME
		ElseIf(button_endgame.hit())
			End;
		EndIf
	End Method
	
	Method draw()
		button_newgame.draw();
		If(FileType("quicksave.sav")=1)
			button_quicksave.draw();
		EndIf
		If(FileType("autosave.sav")=1)
			button_autosave.draw();
		EndIf
		button_endgame.draw();
		
		SetBlend(ALPHABLEND)
		SetColor(255, 2555, 255)
		DrawText("Version 1.0", 4, 600-40)
		DrawText("Graphics and Programming by Stefan Funke", 4, 600-20)
		SetBlend(MASKBLEND)
	End Method
End Type
