Type TControl
	Global buildTemplates:TStructure[];

	Field storage:Int[];
	Field reserved:Int[];
	Field gold:Int;
	Field faction:Int;
	Field state:Int;
	Field buildings:TList;
	Field jobs:TList;
	Method init_control:TControl(gold:Int)
		storage=New Int[TGoods.N_GOODS];
		reserved=New Int[TGoods.N_GOODS];
		Self.gold=gold;
		buildings=New TList;
		jobs=New TList;
		Return Self;
	End Method
	Method update(dt:Double)
	End Method
	Method addObject(obj:TGameObject)
		If(obj)
			obj._link_buildings=ListAddFirst(buildings, obj);
		EndIf
	End Method

	Method getMaxCapacity:Int()
		Local baseCapacity:Int=10;
		Local val:Int=0
		For Local warehouse:TWarehouse=EachIn buildings
			val:+warehouse.amount;
		Next
		Return baseCapacity+val;
	End Method
	
	Function loadBuildTemplates()
		buildTemplates=New TStructure[14]
		buildTemplates[0]=New TStreet.init_TStreet(Null, 5, "Street");
		buildTemplates[1]=New TWarehouse.init_TWarehouse(10, 10, 5).init_THouse(Null, 200, 1, 1, "Warehouse", 22);
		buildTemplates[2]=New TVendor.init_TVendor().init_THouse(Null, 50, 1, 1, "Vendor", 28);
		buildTemplates[3]=New TFruitPicker.init_TFruitPicker(10, 3).init_THouse(Null, 100, 1, 1, "Fruit Picker", 24);
		buildTemplates[4]=New TFarm.init_TFarm(10, 3).init_THouse(Null, 100, 1, 1, "Farm", 27);
		buildTemplates[5]=New TWindMill.init_TWindMill(10, 10, 5, 1).init_THouse(Null, 150, 1, 1, "Windmill", 25);
		buildTemplates[6]=New TCakeBaker.init_TCakeBaker(10, 10, 5, 1).init_THouse(Null, 400, 1, 1, "Cake Factory", 21);
		buildTemplates[7]=New TJuiceFactory.init_TJuiceFactory(10, 10, 5, 1).init_THouse(Null, 400, 1, 1, "Juice Factory", 29);
		buildTemplates[8]=New TFruit.init_TFruit(Null, 5, "Berry Bush", 11, TGoods.GOODS_BERRIES, 1, 0.33, TFruit.PRODUCING);
		buildTemplates[9]=New TFruit.init_TFruit(Null, 5, "Apple Tree", 6, TGoods.GOODS_APPLES, 1, 0.33, TFruit.PRODUCING);
		buildTemplates[10]=New TFruit.init_TFruit(Null, 5, "Cherry Tree", 5, TGoods.GOODS_CHERRIES, 1, 0.33, TFruit.PRODUCING);
		buildTemplates[11]=New TFruit.init_TFruit(Null, 10, "Banana Tree",  4, TGoods.GOODS_BANANAS, 1, 0.33, TFruit.PRODUCING);
		buildTemplates[12]=New TFruit.init_TFruit(Null, 10, "Melon Field", 7, TGoods.GOODS_MELONS, 1, 0.33, TFruit.PRODUCING);
		buildTemplates[13]=New TCrop.init_TCrop(Null, 5, "Crop Field", 2, TGoods.GOODS_CROP, 0.33, TCrop.PRODUCING);

		For Local i:Int=0 To 13
			If(buildTemplates[i])
				buildTemplates[i].noUpdate=True;
			EndIf
		Next
	End Function
End Type

Type TPlayer Extends TControl
	Const NONE:Int=0;
	Const BUILD:Int=1;
	Field tilemap:TTilemap;
	Field buildId:Int;
	Field focusTile:TTile;
	Field markedHouse:THouse;
	Field userInterface:TUserInterface;
	Field holdMouse:Int, mouseUp:Int;
	Field startTile:TTile, endTile:TTile;
	Method init:TPlayer(tilemap:TTileMap, gold:Int)
		Self.tilemap=tilemap;
		init_control(gold);
		state=NONE;
		userInterface=New TUserInterface.init(Self);
		Return Self
	End Method
	Method update(dt:Double)
		For Local j:TJob=EachIn jobs
			j.update(dt);
		Next
		SortList(jobs, True, TJob.compareFunction)
		'mouse focus
		Local focusCoords:Point=New Point;
		TTileMap.getCoords(focusCoords, MouseX()-GameManager.scroll.x, MouseY()-GameManager.scroll.y);
		If(focusCoords.x>=0 And focusCoords.x<tilemap.dimension.x And focusCoords.y>=0 And focusCoords.y<tilemap.dimension.y)
			focusTile=tilemap.data[focusCoords.x, focusCoords.y];
		Else
			focusTile=Null;
		EndIf
		If(focusTile And THouse(focusTile.structure) And MHit1 And MouseX()<=800-112)
			If(Not (state=BUILD And buildId<0))
				markedHouse=THouse(focusTile.structure);
			EndIf
		ElseIf(state=NONE And MHit2)
			markedHouse=Null;
		EndIf
		If(markedHouse And Not markedHouse.active)
			markedHouse=Null
		EndIf
		
		userInterface.update(dt);
		Select(state)
			Case NONE

			Case BUILD
				If(MHit2)
					If(markedHouse)
						markedHouse=Null;
					ElseIf(endTile And startTile)
						holdMouse=False;
						mouseUp=False;
						endTile=Null;
						startTile=Null;		
					Else
						state=NONE;
						If(buildId>=0 And buildId<Len(buildTemplates))
							buildTemplates[buildId].erase();
						EndIf
						FlushKeys()
						Return;
					EndIf
				EndIf
				'If(Not focusTile)
				'	Return;
				'EndIf
				updatePreview(dt);

				If(buildId=-1)
					If(MHit1 And MouseX()<800-112)
						If(focusTile And focusTile.faction=faction)
							focusTile.eraseStructure();
							TStreet.updateId3x3(tilemap, focusTile.coords.x, focusTile.coords.y);
						EndIf
					EndIf
				EndIf
				If(buildId=0)
					If(MHit1 And focusTile And holdMouse=False And MouseX()<800-112)
						holdMouse=True;
						startTile=focusTile;
						endTile=focusTile;
					EndIf
					If(holdMouse And Not MouseDown(1))
						holdMouse=False;
						mouseUp=True;
					EndIf
					If(focusTile And startTile)
						Local cost:Int=0;
						Local direction:Point=New Point;
						If(Abs(focusTile.coords.x-startTile.coords.x)>Abs(focusTile.coords.y-startTile.coords.y))
							direction.x=Sgn(focusTile.coords.x-startTile.coords.x);
						Else
							direction.y=Sgn(focusTile.coords.y-startTile.coords.y);
						EndIf
						Local currentTile:TTile;
						For Local k:Int=0 To Max(Abs(focusTile.coords.x-startTile.coords.x), Abs(focusTile.coords.y-startTile.coords.y))
							currentTile=tilemap.data[startTile.coords.x+direction.x*k, startTile.coords.y+direction.y*k];
							cost:+buildTemplates[buildId].price;
							If(GameManager.player.gold>=cost And buildTemplates[buildId].placementPossible(faction, currentTile.coords.x, currentTile.coords.y))
								endTile=currentTile;
							Else
								Exit;
							EndIf
						Next
					EndIf
					If(mouseUp)
						If(startTile And endTile)
							Local direction:Point=New Point;
							If(Abs(endTile.coords.x-startTile.coords.x)>Abs(endTile.coords.y-startTile.coords.y))
								direction.x=Sgn(endTile.coords.x-startTile.coords.x);
							Else
								direction.y=Sgn(endTile.coords.y-startTile.coords.y);
							EndIf
							Local currentTile:TTile;
							For Local k:Int=0 To Max(Abs(endTile.coords.x-startTile.coords.x), Abs(endTile.coords.y-startTile.coords.y))
								currentTile=tilemap.data[startTile.coords.x+direction.x*k, startTile.coords.y+direction.y*k];
								If(GameManager.player.gold>=buildTemplates[buildId].price And buildTemplates[buildId].placementPossible(faction, currentTile.coords.x, currentTile.coords.y))
									GameManager.player.gold:-buildTemplates[buildId].price;
									currentTile.eraseStructure();
									currentTile.placeStructure(New TStreet.init_TStreet(currentTile, buildTemplates[buildId].price, buildTemplates[buildId].name));
									TStreet.updateId3x3(tilemap, currentTile.coords.x, currentTile.coords.y)
								Else
									Exit;
								EndIf
							Next
						EndIf
						mouseUp=False;
						startTile=Null;
						endTile=Null;
					EndIf
				EndIf
				
				If(buildId>=8)
					If(MHit1 And focusTile And holdMouse=False And MouseX()<800-112)
						holdMouse=True;
						startTile=focusTile;
						endTile=focusTile;
					EndIf
					If(holdMouse And Not MouseDown(1))
						holdMouse=False;
						mouseUp=True;
					EndIf
					If(focusTile And startTile)
						Local cost:Int=0;
						Local currentTile:TTile;
						For Local y:Int=Min(startTile.coords.y, focusTile.coords.y) To Max(startTile.coords.y, focusTile.coords.y)
							For Local x:Int=Min(startTile.coords.x, focusTile.coords.x) To Max(startTile.coords.x, focusTile.coords.x)
								currentTile=tilemap.data[x, y];
								If(buildTemplates[buildId].placementPossible(faction, x, y))
									cost:+buildTemplates[buildId].price;
								EndIf
								If(GameManager.player.gold>=cost)
									endTile=focusTile;
								Else
									Exit;
								EndIf
							Next
						Next
					EndIf
					If(mouseUp)
						If(startTile And endTile)
							Local currentTile:TTile;
							For Local y:Int=Min(startTile.coords.y, endTile.coords.y) To Max(startTile.coords.y, endTile.coords.y)
								For Local x:Int=Min(startTile.coords.x, endTile.coords.x) To Max(startTile.coords.x, endTile.coords.x)
									currentTile=tilemap.data[x, y];
									If(GameManager.player.gold>=buildTemplates[buildId].price And buildTemplates[buildId].placementPossible(faction, currentTile.coords.x, currentTile.coords.y))
										GameManager.player.gold:-buildTemplates[buildId].price;
										Select(buildId)
											Case 8, 9, 10, 11, 12
												Local fr:TFruit=TFruit(buildTemplates[buildId]);
												currentTile.eraseStructure();
												currentTile.placeStructure(New TFruit.init_TFruit(currentTile, fr.price, fr.name, fr.frame, fr.fruitId, Len(fr.fruits), fr.rate));
											Case 13
												Local cr:TCrop=TCrop(buildTemplates[buildId]);
												currentTile.eraseStructure();
												currentTile.placeStructure(New TCrop.init_TCrop(currentTile, cr.price, cr.name, cr.frame, cr.fruitId, cr.rate));
										End Select
										
									Else
										'Exit;
									EndIf
								Next
							Next
						EndIf
						mouseUp=False;
						startTile=Null;
						endTile=Null;
					EndIf
				EndIf
				
				If((buildId>0 And buildId<=7))
					If(MHit1 And MouseX()<800-112)
						If(focusTile And GameManager.player.gold>=buildTemplates[buildId].price And buildTemplates[buildId].placementPossible(faction, focusTile.coords.x, focusTile.coords.y))
							GameManager.player.gold:-buildTemplates[buildId].price;
							Select(buildId)
								Case 1
									Local wh:TWarehouse=TWarehouse(buildTemplates[buildId]);
									Local new_wh:THouse=New TWarehouse.init_THouse(GameManager.tilemap.data[focusTile.coords.x, focusTile.coords.y], wh.price, wh.dx, wh.dy, wh.name, wh.frame);
									TWarehouse(new_wh).init_TWarehouse(wh.amount, wh.areaRange, Len(wh.workers));
									TWarehouse(new_wh).initArea(faction);
									GameManager.tilemap.data[focusTile.coords.x, focusTile.coords.y].eraseStructure();
									GameManager.tilemap.data[focusTile.coords.x, focusTile.coords.y].placeStructure(new_wh);
									'GameManager.player.addObject(new_wh);
								Case 3
									Local fp:TFruitPicker=TFruitPicker(buildTemplates[buildId]);
									Local new_fp:THouse=New TFruitPicker.init_THouse(GameManager.tilemap.data[focusTile.coords.x, focusTile.coords.y], fp.price, fp.dx, fp.dy, fp.name, fp.frame);
									TFruitPicker(new_fp).init_TFruitPicker(fp.amount, fp.areaRange, Len(fp.workers));
									GameManager.tilemap.data[focusTile.coords.x, focusTile.coords.y].eraseStructure();
									GameManager.tilemap.data[focusTile.coords.x, focusTile.coords.y].placeStructure(new_fp);
									'GameManager.player.addObject(new_fp);
								Case 6
									Local cb:TCakeBaker=TCakeBaker(buildTemplates[buildId]);
									Local new_cb:THouse=New TCakeBaker.init_THouse(GameManager.tilemap.data[focusTile.coords.x, focusTile.coords.y], cb.price, cb.dx, cb.dy, cb.name, cb.frame);
									TCakeBaker(new_cb).init_TCakeBaker(cb.amount, cb.areaRange, cb.rate, Len(cb.workers));
									GameManager.tilemap.data[focusTile.coords.x, focusTile.coords.y].eraseStructure();
									GameManager.tilemap.data[focusTile.coords.x, focusTile.coords.y].placeStructure(new_cb);
									'GameManager.player.addObject(new_cb);
								Case 4
									Local farm:TFarm=TFarm(buildTemplates[buildId]);
									Local new_farm:THouse=New TFarm.init_THouse(GameManager.tilemap.data[focusTile.coords.x, focusTile.coords.y], farm.price, farm.dx, farm.dy, farm.name, farm.frame);
									TFarm(new_farm).init_TFarm(farm.amount, farm.areaRange, Len(farm.workers));
									GameManager.tilemap.data[focusTile.coords.x, focusTile.coords.y].eraseStructure();
									GameManager.tilemap.data[focusTile.coords.x, focusTile.coords.y].placeStructure(new_farm);
									'GameManager.player.addObject(new_farm);
								Case 5
									Local wm:TWindMill=TWindMill(buildTemplates[buildId]);
									Local new_wm:THouse=New TWindMill.init_THouse(GameManager.tilemap.data[focusTile.coords.x, focusTile.coords.y], wm.price, wm.dx, wm.dy, wm.name, wm.frame);
									TWindMill(new_wm).init_TWindMill(wm.amount, wm.areaRange, wm.rate, Len(wm.workers));
									GameManager.tilemap.data[focusTile.coords.x, focusTile.coords.y].eraseStructure();
									GameManager.tilemap.data[focusTile.coords.x, focusTile.coords.y].placeStructure(new_wm);
									'GameManager.player.addObject(new_wm);
								Case 2
									Local vend:TVendor=TVendor(buildTemplates[buildId]);
									Local new_vend:THouse=New TVendor.init_THouse(GameManager.tilemap.data[focusTile.coords.x, focusTile.coords.y], vend.price, vend.dx, vend.dy, vend.name, vend.frame);
									TVendor(new_vend).init_TVendor();
									GameManager.tilemap.data[focusTile.coords.x, focusTile.coords.y].eraseStructure();
									GameManager.tilemap.data[focusTile.coords.x, focusTile.coords.y].placeStructure(new_vend);
									'GameManager.player.addObject(new_vend);
								Case 7
									Local jf:TJuiceFactory=TJuiceFactory(buildTemplates[buildId]);
									Local new_jf:THouse=New TJuiceFactory.init_THouse(GameManager.tilemap.data[focusTile.coords.x, focusTile.coords.y], jf.price, jf.dx, jf.dy, jf.name, jf.frame);
									TJuiceFactory(new_jf).init_TJuiceFactory(jf.amount, jf.areaRange, jf.rate, Len(jf.workers));
									GameManager.tilemap.data[focusTile.coords.x, focusTile.coords.y].eraseStructure();
									GameManager.tilemap.data[focusTile.coords.x, focusTile.coords.y].placeStructure(new_jf);
									'GameManager.player.addObject(new_jf);
							End Select

							'state=NONE
						EndIf
					EndIf
				EndIf
		End Select
	End Method
	Method updatePreview(dt:Double)
		If(buildId>=0 And buildId<Len(buildTemplates) And focusTile)
			'blink when insufficient money
			If(GameManager.player.gold<buildTemplates[buildId].price)
				TStructure.blink_timer:-dt;
				If(TStructure.blink_timer<=0)
					TStructure.blink=Not TStructure.blink;
					TStructure.blink_timer=0.5;
				EndIf
			Else
				TStructure.blink=False;
			EndIf
			'update position of preview
			If(buildId>0)
				buildTemplates[buildId].pos.x=focusTile.pos.x;
				buildTemplates[buildId].pos.y=focusTile.pos.y;
				buildTemplates[buildId].tile=focusTile;
				If(buildId>0 And buildId <> 13)
					buildTemplates[buildId].pos.y:+8;
				EndIf
			Else
				If(endTile)
					buildTemplates[buildId].pos.x=endTile.pos.x;
					buildTemplates[buildId].pos.y=endTile.pos.y;
					buildTemplates[buildId].tile=endTile;
				Else
					buildTemplates[buildId].pos.x=focusTile.pos.x;
					buildTemplates[buildId].pos.y=focusTile.pos.y;
					buildTemplates[buildId].tile=focusTile;
				EndIf
			EndIf
			TTileMap.getCoords(buildTemplates[buildId].sortCoords, buildTemplates[buildId].pos.x, buildTemplates[buildId].pos.y);
			If(TFruit(buildTemplates[buildId]))
				For Local fr:TTree=EachIn TFruit(buildTemplates[buildId]).fruits
					fr.tile=buildTemplates[buildId].tile;
					fr.pos.x=buildTemplates[buildId].pos.x;
					fr.pos.y=buildTemplates[buildId].pos.y;
					TTileMap.getCoords(fr.sortCoords, fr.pos.x, fr.pos.y);
				Next
			EndIf
		EndIf
	End Method
	Method drawUI(scroll:Point)
		userInterface.draw();
	End Method

	Method saveToFile(out:TStream)
		writeintArray(out, storage);
		writeIntArray(out, reserved);
		WriteInt(out, gold);
		WriteInt(out, faction);
		Rem
		WriteInt(out, CountList(jobs));
		For Local job:TJob=EachIn jobs
			job.saveToFile(out);
		Next
		End Rem
	End Method
	
	Method loadFromFile(in:TStream)
		storage=readIntArray(in);
		reserved=readIntArray(in);
		gold=ReadInt(in);
		faction=ReadInt(in);
		Rem
		Local n:Int=ReadInt(in);
		'jobs=New TList;
		Local job:TJob;
		For Local i:Int=0 To n-1
			job=New TJob;
			job.loadFromFile(in);
			If(job.structure)
				ListAddLast(jobs, job);
			Else
			EndIf
		Next
		End Rem
	End Method
End Type
