Type TTile Extends TGameObject
	Field coords:Point;
	Field collision:Int;
	Field structure:TStructure;
	Field faction:Int;
	'Field productivity:Int[];
	Method init_TTile:TTile(x:Int, y:Int, frame:Int=0, coll:Int=False)
		Super.init(TTileMap.getPos(x, y), TResManager.IMG_TERRAIN, frame)
		coords=New Point.init(x, y);
		collision=coll;
'		productivity=New Int[TGoods.N_GOODS];
'		For Local i:Int=0 To TGoods.N_GOODS-1
'			productivity[i]=1;
'		Next
		Return Self
	End Method
	Method placeStructure(s:TStructure)
		If(Not structure And Not collision)
			structure=s;
			If(Not TStreet(structure))
				GameManager.addObject(structure);
			EndIf
			If(THouse(structure))
				GameManager.players[structure.faction].addObject(structure);
			EndIf
		EndIf
	End Method
	Method eraseStructure()
		If(structure)
			structure.erase();
		EndIf
		structure=Null;
	End Method
	Method draw(scroll:Point)
		If(onScreen(scroll))
			Super.draw(scroll);
		EndIf
	End Method
	
	Method saveToFile(out:TStream)
		Super.saveToFile(out);
		WriteInt(out, collision);
		WriteInt(out, faction);
	End Method
	Method loadFromFile(in:TStream)
		Super.loadFromFile(in);
		collision=ReadInt(in);
		faction=ReadInt(in);
	End Method
End Type


Type TTileMap
	Const TILESIZE:Int=32;
	Field dimension:Point;
	Field data:TTile[, ];

	Method init:TTileMap(x:Int, y:Int)
		dimension=New Point.init(x, y);
		data=New TTile[x, y];
		For Local x0:Int=0 To dimension.x-1
			For Local y0:Int=0 To dimension.y-1
				data[x0, y0]=New TTile.init_TTile(x0, y0);
			Next
		Next
		Return Self;
	End Method

	
	Method draw(scroll:Point)
		For Local x:Int=0 To dimension.x-1
			For Local y:Int=0 To dimension.y-1
				If(data[x, y].onScreen(scroll))
					data[x, y].setDraw(scroll);
					data[x, y].draw(scroll);
					If(data[x, y].structure And (data[x, y].structure.getType()=TStructure.STRUCT_STREET Or data[x, y].structure.getType()=TStructure.STRUCT_CROP))
						data[x, y].structure.setDraw(scroll);
						data[x, y].structure.draw(scroll);
					EndIf
				EndIf
			Next
		Next
	End Method
	
	Function getPos:Point(x:Int, y:Int)
		Local p:Point=New Point;
		p.x=(x-y)*TILESIZE/2.0;
		p.y=(x+y)*TILESIZE/4.0;
		Return p
	End Function
	Function inRange:Int(p1:Point, p2:Point, range:Int)
		Local eps:Double=0.001;
		Local phi:Double=ATan2(p2.y-p1.y, p2.x-p1.x);
		If(Point.distance(p1, p2) <= (New Point.init(TTileMap.TILESIZE*range*Cos(phi), TTileMap.TILESIZE*range*0.5*Sin(phi))).length() + eps)
			Return True;
		EndIf
		Return False;
	End Function
	Function inRangeSup:Int(p1:Point, p2:Point, range:Int)
		Local eps:Double=0.001;
		If(Abs(p1.x-p2.x)<=range And Abs(p1.y-p2.y)<=range)
			Return True;
		EndIf
		Return False;
	End Function
	Function getCoords(p:Point, x:Int, y:Int)
		'Local p:Point= New Point;
		If(p=Null)
			Return;
		EndIf
		p.x = Floor(y*2.0/TTileMap.TILESIZE + x/Double(TTileMap.TILESIZE));
		p.y = Floor(y*2.0/TTileMap.TILESIZE - x/Double(TTileMap.TILESIZE));
		'Return p
	End Function
	Function onTileMap:Int(dimension:Point, x:Int, y:Int)
		If(x>=0 And x<dimension.x And y>=0 And y<dimension.y)
			Return True;
		EndIf
		Return False;
	End Function
	Function getCoordsDouble2:Point(x:Int, y:Int)
		Local p:Point= New Point;
		p.x = (y*2.0/TTileMap.TILESIZE + x/Double(TTileMap.TILESIZE));
		p.y = (y*2.0/TTileMap.TILESIZE - x/Double(TTileMap.TILESIZE));
		Return p
	End Function
	Function AStar:TList(startHouse:THouse, endTile:TTile)
		Local startTile:TTile=startHouse.tile;
		Local closedSet:TList=New TList;
		Local openSet:TList=New TList;
		Local startNode:TNode=New TNode.init(startTile)
		startNode.g=0;
		startNode.h=startNode.get_h(endTile);
		startNode.f=startNode.g+startNode.h;
		ListAddFirst(openSet, startNode);
		
		Local currentNode:TNode;
		While(Not ListIsEmpty(openSet))
			SortList(openSet, False, TNode.compareFunction);
			currentNode=TNode(openSet.first());
			If(currentNode.tile=endTile)
				Local path:TList=New TList;
				Repeat
					ListAddFirst(path, currentNode.tile);
					currentNode=currentNode.previousNode;
				Until((currentNode=Null));
				Return path;
			EndIf
			ListRemove(openSet, currentNode);
			ListAddFirst(closedSet, currentNode);
			For Local nextNode:TNode=EachIn currentNode.getNextNodes(startHouse, endTile)
				If(TNode.listContainsNode(closedSet, nextNode))
					Continue;
				EndIf
				Local g:Int=currentNode.g+nextNode.local_g;
				Local node:TNode=TNode.listContainsNode(openset, nextNode);
				If(Not node)
					nextNode.previousNode=currentNode;
					nextNode.h=nextNode.get_h(endTile);
					nextNode.g=g;
					nextNode.f=nextNode.g+nextNode.h;
					ListAddFirst(openset, nextNode);
				ElseIf(g < node.g)
					node.previousNode=currentNode;
					node.g=g;
					node.f=node.g+node.h				
				EndIf
			Next
		Wend
		Return Null;
	End Function
End Type

Type TNode
	Field tile:TTile;
	Field previousNode:TNode;
	Field g:Int, h:Int, f:Int;
	Field local_g:Int;
	Method init:TNode(tile:TTile, loc_g:Int=1)
		Self.tile=tile;
		local_g=loc_g;
		Return Self;
	End Method
	Method get_h:Int(endTile:TTile)
		Return Abs(endTile.coords.x-tile.coords.x)+Abs(endTile.coords.y-tile.coords.y);
	End Method
	Method getNextNodes:TList(house:THouse, endTile:TTile)
		Local nextNodes:TList=New TList;
		Local node:TNode;
		node=getNextNode(house, tile.coords.x+1, tile.coords.y, endTile);
		If(node)
			ListAddLast(nextNodes, node);
		EndIf
		node=getNextNode(house, tile.coords.x-1, tile.coords.y, endTile);
		If(node)
			ListAddLast(nextNodes, node);
		EndIf
		node=getNextNode(house, tile.coords.x, tile.coords.y+1, endTile);
		If(node)
			ListAddLast(nextNodes, node);
		EndIf
		node=getNextNode(house, tile.coords.x, tile.coords.y-1, endTile);
		If(node)
			ListAddLast(nextNodes, node);
		EndIf
		Return nextNodes;
	End Method
	Method getNextNode:TNode(house:THouse, x0:Int, y0:Int, endTile:TTile)
		If(TTileMap.onTileMap(GameManager.tilemap.dimension, x0, y0))
			If(TTileMap.inRangeSup(house.coords, New point.init(x0, y0), house.areaRange) And (TStreet(GameManager.tilemap.data[x0, y0].Structure) Or TTownHouse(GameManager.tilemap.data[x0, y0].Structure) Or GameManager.tilemap.data[x0, y0]=endTile))
				If(TTownHouse(GameManager.tilemap.data[x0, y0].Structure))
					Return (New TNode.init(GameManager.tilemap.data[x0, y0], 5));
				Else
					Return (New TNode.init(GameManager.tilemap.data[x0, y0]));
				EndIf
			EndIf
		EndIf
		Return Null
	End Method
	Function listContainsNode:TNode(list:TList, node:TNode)
		For Local n:Tnode=EachIn list
			If(n.tile=node.tile)
				Return n;
			EndIf
		Next
		Return Null;
	End Function
	Function compareFunction:Int(obj1:Object, obj2:Object)
		If(TNode(obj1) And TNode(obj2))
			If(TNode(obj1).f < TNode(obj2).f)
				Return 1;
			Else
				Return -1;
			EndIf
		EndIf
		Return 0;
	End Function
End Type

