Type TWorker Extends TGameObject
	Const FR_WALK:Int=0, FR_CARRY:Int=2, FR_PICK:Int=4, FR_CROP:Int=6;
	Field state:Int
	Field v:Double;
	Field actionQueue:TList
	Field currentAction:TAction
	Field job:TJob;
	Field carryAmount:Int, carryId:Int, maxCarry:Int;
	Field direction:Int;
	Field animTimer:Double, animDuration:Double;
	Field dFrame:Int, nFrames:Int;
	Field gender:Int;
	Field productivity:Double, work_timer:Double, total_timer:Double;
	Field house:THouse;
	
	Method init_TWorker:TWorker(pos:Point, v:Double, gender:Int, maxCarry:Int, house:THouse)
		Super.init(pos, TResManager.IMG_WORKER, 0)
		Self.gender=gender;
		Self.v=v;
		Self.maxCarry=maxCarry;
		Self.house=house;
		actionQueue=New TList;
		direction=1;
		animTimer=0.4; animDuration=0.4;
		dFrame=0; nFrames=2;
		Return Self;
	End Method
	Method update(dt:Double)
		total_timer:+dt;
		If(currentAction)
			work_timer:+dt;
			currentAction.update(dt);
			If(currentAction.finished)
				'RemoveLink(currentAction._link);
				currentAction=Null;
			EndIf
		EndIf
		If(Not currentAction)
			If(Not ListIsEmpty(actionQueue))
				currentAction=TAction(actionQueue.removeFirst());
				If(currentAction)
					currentAction.setAnimation();
				EndIf
			EndIf
		EndIf
		
		If(Not currentAction)
			If(house=Null)
				active=False;
			EndIf
		EndIf
		
		animTimer:-dt;
		If(animTimer<=0)
			animTimer=animDuration;
			dFrame = (dFrame+1) Mod nFrames;
		EndIf
		If(total_timer>=5)
			productivity=(0.8*productivity+0.2*work_timer/total_timer);
			work_timer=0;
			total_timer=0;
		EndIf
	End Method
	Method draw(scroll:Point)
		If(currentAction And frame>=0)
			If(onScreen(scroll))
				SetScale(direction, 1);
				DrawImage(TResManager.img[imgId], Floor(pos.x+scroll.x), Floor(pos.y+scroll.y), (frame+dFrame)+gender*16);
				SetScale(1, 1);
			EndIf
		EndIf
	End Method
	Method getTrueSpeed:Double()
		Local maxCarry:Int=5;
		Return v*(1-Min(carryAmount, maxCarry)*0.05);
	End Method
	Method addAction(action:TAction)
		action._link=ListAddLast(actionQueue, action)
	End Method
	
	Method saveToFile(out:TStream)
		Super.saveToFile(out);
		WriteInt(out, house.coords.x);
		WriteInt(out, house.coords.y);
		WriteInt(out, state);
		WriteDouble(out, v);
		If(currentAction)
			WriteInt(out, 1);
			WriteInt(out, currentAction.getId());
			currentAction.saveToFile(out);
		Else
			WriteInt(out, 0);
		EndIf
		WriteInt(out, CountList(actionqueue));
		For Local action:TAction=EachIn actionqueue
			'save actionId
			WriteInt(out, action.getId());
			action.saveToFile(out);
		Next
		If(job)
			WriteInt(out, 1);
			job.saveToFile(out);
		Else
			WriteInt(out, 0);
		EndIf
		WriteInt(out, carryAmount);
		WriteInt(out, carryId);
		WriteInt(out, maxCarry);
		WriteInt(out, direction);
		WriteDouble(out, animTimer);
		WriteDouble(out, animDuration);
		WriteInt(out, dFrame);
		WriteInt(out, nFrames);
		WriteInt(out, gender);
		WriteDouble(out, productivity);
		WriteDouble(out, work_timer);
		WriteDouble(out, total_timer);
		Local saveId:Int=-1;
		For Local i:Int=0 To Len(house.workers)-1
			If(house.workers[i]=Self)
				saveId=i;
				Exit
			EndIf
		Next
		If(saveId>=0)
			WriteInt(out, saveId);
		Else
			WriteInt(out, -1);
		EndIf
	End Method
	Method loadFromFile(in:TStream)
		Super.loadFromFile(in);
		Local x:Int=ReadInt(in);
		Local y:Int=ReadInt(in);
		If(THouse(GameManager.tilemap.data[x, y].structure))
			house=THouse(GameManager.tilemap.data[x, y].structure)
		Else
			house=Null;
		EndIf
		state=ReadInt(in);
		v=ReadDouble(in);
		'currentaction
		Local curr:Int=ReadInt(in);
		Local action:TAction=Null;
		If(curr)
			Local id:Int=ReadInt(in)
			Select(id)
				Case 1
					action=New TAction_Spawn;
				Case 2
					action=New TAction_WalkTo;
				Case 3
					action=New TAction_pickFruit;
				Case 4
					action=New TAction_pickCrop;
				Case 5
					action=New TAction_rest;
				Case 6
					action=New TAction_put;
				Case 7
					action=New TAction_take;
				Case 8
					action=New TAction_walkPath;
				Case 9
					action=New TAction_buy;
			End Select
			If(action)
				action.loadFromFile(in);
				action.worker=Self;
			EndIf
		EndIf
		currentAction=action;
		'actionlist
		Local n:Int=ReadInt(in);
		actionqueue=New TList;
		Local id:Int;
		For Local i:Int=0 To n-1
			id=ReadInt(in)
			Local action:TAction;
			Select(id)
				Case 1
					action=New TAction_Spawn;
				Case 2
					action=New TAction_WalkTo;
				Case 3
					action=New TAction_pickFruit;
				Case 4
					action=New TAction_pickCrop;
				Case 5
					action=New TAction_rest;
				Case 6
					action=New TAction_put;
				Case 7
					action=New TAction_take;
				Case 8
					action=New TAction_walkPath;
				Case 9
					action=New TAction_buy;
			End Select
			If(action)
				action.loadFromFile(in);
				action.worker=Self;
				action._link=ListAddLast(actionqueue, action);
			EndIf
		Next
		Local hasJob:Int=ReadInt(in);
		If(hasJob)
			job=New TJob;
			job.loadFromFile(in);
		EndIf
		carryAmount=ReadInt(in);
		carryId=ReadInt(in);
		maxCarry=ReadInt(in);
		direction=ReadInt(in);
		animTimer=ReadDouble(in);
		animDuration=ReadDouble(in);
		dFrame=ReadInt(in);
		nFrames=ReadInt(in);
		gender=ReadInt(in);
		productivity=ReadDouble(in);
		work_timer=ReadDouble(in);
		total_timer=ReadDouble(in);
		Local workerId:Int=ReadInt(in);
		If(house And workerId>=0)
			house.workers[workerId]=Self;
		EndIf
	End Method
End Type

Type TAction
	Const SPAWN:Int=1, WALKTO:Int=2, PICKFRUIT:Int=3, PICKCROP:Int=4, REST:Int=5, PUT:Int=6, TAKE:Int=7, WALKPATH:Int=8, BUY:Int=9;
	Field _link:TLink
	Field worker:TWorker
	Field finished:Int
	Method update(dt:Double)
	End Method
	Method setAnimation()
		worker.frame=-1;
	End Method
	Method getId:Int()
		Return 0;
	End Method
	Method saveToFile(out:TStream)
		WriteInt(out, finished);
	End Method
	Method loadFromFile(in:TStream)
		finished=ReadInt(in);
	End Method
End Type

Type TAction_Spawn Extends TAction
	Field location:Point;
	Method init:TAction_Spawn(worker:TWorker, loc:Point)
		Self.worker=worker;
		location=loc.copy();
		Return Self
	End Method
	Method update(dt:Double)
		worker.pos=location;
		TTileMap.getCoords(worker.sortCoords, worker.pos.x, worker.pos.y);
		finished=True;
	End Method
	Method getId:Int()
		Return SPAWN;
	End Method
	Method saveToFile(out:TStream)
		Super.saveToFile(out);
		WriteDouble(out, location.x);
		WriteDouble(out, location.y);
	End Method
	Method loadFromFile(in:TStream)
		Super.loadFromFile(in);
		location=New Point;
		location.x=ReadDouble(in);
		location.y=ReadDouble(in);
	End Method
End Type

Type TAction_WalkTo Extends TAction
	Field moveTarget:Point;
	Method init:TAction_WalkTo(worker:TWorker, target:Point)
		moveTarget=target.copy();
		Self.worker=worker;
		Return Self
	End Method
	Method update(dt:Double)
		If(moveTarget.x<worker.pos.x)
			worker.direction=-1;
		Else
			worker.direction=1;
		EndIf
		Local dist:Double=Point.distance(moveTarget, worker.pos);
		worker.pos.x:+worker.getTrueSpeed()*dt*(moveTarget.x-worker.pos.x)/dist;
		worker.pos.y:+worker.getTrueSpeed()*dt*(moveTarget.y-worker.pos.y)/dist;
		If(Point.distance(moveTarget, worker.pos)<dt*worker.getTrueSpeed()+0.001)
			worker.pos.x=moveTarget.x;
			worker.pos.y=moveTarget.y;
			finished=True;
		EndIf
		TTileMap.getCoords(worker.sortCoords, worker.pos.x, worker.pos.y);
	End Method
	Method setAnimation()
		If(worker.carryAmount=0)
			worker.frame=TWorker.FR_WALK;
		Else
			worker.frame=TWorker.FR_CARRY;
		EndIf
	End Method
	Method getId:Int()
		Return WALKTO;
	End Method
	Method saveToFile(out:TStream)
		Super.saveToFile(out);
		WriteDouble(out, moveTarget.x);
		WriteDouble(out, moveTarget.y);
	End Method
	Method loadFromFile(in:TStream)
		Super.loadFromFile(in);
		moveTarget=New Point;
		moveTarget.x=ReadDouble(in);
		moveTarget.y=ReadDouble(in);
	End Method
End Type

Type TAction_pickFruit Extends TAction
	Field fruit:TFruit;
	Field duration:Double;
	Method init:TAction_pickFruit(worker:TWorker, duration:Double, fruit:TFruit)
		Self.worker=worker;
		Self.fruit=fruit;
		Self.duration=duration;
		Return Self
	End Method
	Method update(dt:Double)
		duration:-dt;
		If(duration<=0 Or (Not fruit) Or (fruit.active=False))
			If(fruit)
				Local amount:Int=Min(Floor(fruit.amount), worker.maxCarry-worker.carryAmount)
				worker.carryAmount:+amount;
				worker.carryId=fruit.fruitId;
				fruit.amount:-amount;
			EndIf
			finished=True;
		EndIf
		If(fruit And fruit.pos.x<worker.pos.x)
			worker.direction=-1;
		Else
			worker.direction=1;
		EndIf
	End Method
	Method setAnimation()
		worker.frame=TWorker.FR_PICK;
	End Method
	Method getId:Int()
		Return PICKFRUIT;
	End Method
	Method saveToFile(out:TStream)
		Super.saveToFile(out);
		If(fruit)
			WriteInt(out, fruit.coords.x);
			WriteInt(out, fruit.coords.y);
		Else
			WriteInt(out, -1);
			WriteInt(out, -1);
		EndIf
		WriteDouble(out, duration);
	End Method
	Method loadFromFile(in:TStream)
		Super.loadFromFile(in);
		Local x:Int=ReadInt(in);
		Local y:Int=ReadInt(in);
		If(x>=0 And y>=0 And TFruit(GameManager.tilemap.data[x, y].structure))
			fruit=TFruit(GameManager.tilemap.data[x, y].structure);
		Else
			fruit=Null;
		EndIf
		duration=ReadDouble(in);
	End Method
End Type

Type TAction_pickCrop Extends TAction
	Field crop:TCrop;
	Field duration:Double;
	Method init:TAction_pickCrop (worker:TWorker, duration:Double, crop:TCrop)
		Self.worker=worker;
		Self.crop=crop;
		Self.duration=duration;
		Return Self
	End Method
	Method update(dt:Double)
		duration:-dt;
		If(duration<=0 Or (Not crop) Or (crop.active=False))
			If(crop)
				Local amount:Int=Min(Floor(crop.amount), worker.maxCarry-worker.carryAmount)
				worker.carryAmount:+amount;
				worker.carryId=TGoods.GOODS_CROP;
				crop.amount:-amount;
			EndIf
			finished=True;
		EndIf
		If(crop And crop.pos.x<worker.pos.x)
			worker.direction=-1;
		Else
			worker.direction=1;
		EndIf
	End Method
	Method setAnimation()
		worker.frame=TWorker.FR_CROP;
	End Method
	Method getId:Int()
		Return PICKCROP;
	End Method
	Method saveToFile(out:TStream)
		Super.saveToFile(out);
		If(crop)
			WriteInt(out, crop.coords.x);
			WriteInt(out, crop.coords.y);
		Else
			WriteInt(out, -1);
			WriteInt(out, -1);
		EndIf
		WriteDouble(out, duration);
	End Method
	Method loadFromFile(in:TStream)
		Super.loadFromFile(in);
		Local x:Int=ReadInt(in);
		Local y:Int=ReadInt(in);
		If(x>=0 And y>=0 And TCrop(GameManager.tilemap.data[x, y].structure))
			crop=TCrop(GameManager.tilemap.data[x, y].structure);
		Else
			crop=Null;
		EndIf
		duration=ReadDouble(in);
	End Method
End Type

Type TAction_rest Extends TAction
	Field duration:Double;
	Method init:TAction_rest(worker:TWorker, duration:Double)
		Self.worker=worker;
		Self.duration=duration;
		Return Self
	End Method
	Method update(dt:Double)
		duration:-dt;
		If(duration<=0)
			finished=True;
		EndIf
	End Method
	Method getId:Int()
		Return REST;
	End Method
	Method saveToFile(out:TStream)
		Super.saveToFile(out);
		WriteDouble(out, duration);
	End Method
	Method loadFromFile(in:TStream)
		Super.loadFromFile(in);
		duration=ReadDouble(in);
	End Method
End Type

Type TAction_Put Extends TAction
	Field goods:Int[];
	Field house:THouse;
	Method init:TAction_Put(worker:TWorker, goods:Int[], house:THouse)
		Self.worker=worker;
		Self.goods=goods;
		Self.house=house;
		Return Self
	End Method
	Method update(dt:Double)
		If(goods And house)
			Local amount:Int=Max(0, Min(worker.carryAmount, GameManager.players[house.faction].getMaxCapacity()-goods[worker.carryId]));
			goods[worker.carryId]:+amount;
			worker.carryAmount:-amount;
		Else
			worker.carryAmount=0;
		EndIf
		finished=True;
	End Method
	Method getId:Int()
		Return PUT;
	End Method
	Method saveToFile(out:TStream)
		Super.saveToFile(out);
		WriteInt(out, house.coords.x);
		WriteInt(out, house.coords.y);
	End Method
	Method loadFromFile(in:TStream)
		Super.loadFromFile(in);
		Local x:Int=ReadInt(in);
		Local y:Int=ReadInt(in);
		If(TWareHouse(GameManager.tilemap.data[x, y].structure))
			house=THouse(GameManager.tilemap.data[x, y].structure);
			goods=GameManager.players[house.faction].storage;
		ElseIf(THouse(GameManager.tilemap.data[x, y].structure))
			house=THouse(GameManager.tilemap.data[x, y].structure);
			goods=house.goods;
		Else
			goods=Null;
			house=Null;
		EndIf
	End Method
End Type

Type TAction_Take Extends TAction
	Field goods:Int[];
	Field fruitId:Int;
	Field house:THouse;
	Method init:TAction_Take(worker:TWorker, fruitId:Int, goods:Int[], house:THouse)
		Self.worker=worker;
		Self.fruitId=fruitId;
		Self.goods=goods;
		Self.house=house;
		Return Self
	End Method
	Method update(dt:Double)
		If(goods)
			Local amount:Int=Min(goods[fruitId], worker.maxCarry-worker.carryAmount)
			worker.carryAmount:+amount;
			worker.carryId=fruitId;
			goods[fruitId]:-amount;
		EndIf
		If(worker.job)
			worker.job.finished();
			worker.job=Null;
		EndIf
		finished=True;
	End Method
	Method getId:Int()
		Return TAKE;
	End Method
	Method saveToFile(out:TStream)
		Super.saveToFile(out);
		WriteInt(out, house.coords.x);
		WriteInt(out, house.coords.y);
		WriteInt(out, fruitId);
	End Method
	Method loadFromFile(in:TStream)
		Super.loadFromFile(in);
		Local x:Int=ReadInt(in);
		Local y:Int=ReadInt(in);
		If(TWareHouse(GameManager.tilemap.data[x, y].structure))
			house=THouse(GameManager.tilemap.data[x, y].structure);
			goods=GameManager.players[house.faction].storage;
		ElseIf(THouse(GameManager.tilemap.data[x, y].structure))
			house=THouse(GameManager.tilemap.data[x, y].structure);
			goods=house.goods;
		Else
			house=Null;
			goods=Null;
		EndIf
		fruitId=ReadInt(in);
	End Method
End Type

Type TAction_WalkPath Extends TAction
	Field targetTile:TTile;
	Field moveTarget:Point;
	Field path:TList;
	Method init:TAction_WalkPath(worker:TWorker, path:TList)
		Self.worker=worker;
		Self.path=path;
		path.removeFirst();
		targetTile=TTile(path.removeFirst());
		If(targetTile)
			moveTarget=targetTile.pos.copy();
			moveTarget.y:+8;
		EndIf
		Return Self
	End Method
	Method update(dt:Double)
		If(moveTarget.x<worker.pos.x)
			worker.direction=-1;
		Else
			worker.direction=1;
		EndIf
		Local dist:Double=Point.distance(moveTarget, worker.pos);
		worker.pos.x:+worker.getTrueSpeed()*dt*(moveTarget.x-worker.pos.x)/dist;
		worker.pos.y:+worker.getTrueSpeed()*dt*(moveTarget.y-worker.pos.y)/dist;
		If(Point.distance(moveTarget, worker.pos)<dt*worker.getTrueSpeed()+0.001)
			'worker.pos.x=moveTarget.x;
			'worker.pos.y=moveTarget.y;
			If(Not ListIsEmpty(path))
				targetTile=TTile(path.removeFirst());
				If(targetTile)
					moveTarget=targetTile.pos.copy();
					moveTarget.y:+8;
				EndIf
			Else
				targetTile=Null;
			EndIf
		EndIf
		If(Not targetTile)
			finished=True;
		EndIf
		TTileMap.getCoords(worker.sortCoords, worker.pos.x, worker.pos.y);
	End Method
	Method setAnimation()
		If(worker.carryAmount=0)
			worker.frame=TWorker.FR_WALK;
		Else
			worker.frame=TWorker.FR_CARRY;
		EndIf
	End Method
	Method getId:Int()
		Return WALKPATH;
	End Method
	Method saveToFile(out:TStream)
		Super.saveToFile(out);
		WriteInt(out, targetTile.coords.x);
		WriteInt(out, targetTile.coords.y);
		WriteDouble(out, moveTarget.x);
		WriteDouble(out, moveTarget.y);
		WriteInt(out, CountList(path));
		For Local tile:TTile=EachIn path
			WriteInt(out, tile.coords.x);
			WriteInt(out, tile.coords.y);
		Next
	End Method
	Method loadFromFile(in:TStream)
		Super.loadFromFile(in);
		Local x:Int=ReadInt(in);
		Local y:Int=ReadInt(in);
		targetTile=GameManager.tilemap.data[x, y];
		Local xd:Double=ReadDouble(in);
		Local yd:Double=ReadDouble(in);
		moveTarget=New Point;
		moveTarget.init(xd, yd);
		Local n:Int=ReadInt(in);
		path=New TList;
		For Local i:Int=0 To n-1
			x=ReadInt(in);
			y=ReadInt(in);
			ListAddLast(path, GameManager.tilemap.data[x, y]);
		Next

	End Method
End Type

Type TAction_Buy Extends TAction
	Field control:TControl;
	Field townhouse:TTownHouse;
	Field duration:Double;
	Field invisible:Int;
	Method init:TAction_Buy(worker:TWorker, control:TControl, destination:TTownHouse, duration:Double=3, invis:Int=False)
		Self.control=control;
		Self.townhouse=destination;
		Self.worker=worker;
		Self.duration=duration;
		invisible=invis;
		Return Self
	End Method
	Method update(dt:Double)
		duration:-dt;
		If(duration<=0 And townhouse)
			Local amount:Int=0;
			For Local i:Int=0 To TGoods.N_GOODS-1
				If(townhouse.rates[i]>0 And townhouse.goods[i]<townhouse.rates[i]*30)
					amount=Min(control.storage[i], Min(Int(townhouse.rates[i]*30)-townhouse.goods[i], townhouse.reserved[i]))'control.storage[i]);
					townhouse.goods[i]:+amount
					control.storage[i]:-amount
					control.gold:+GameManager.world.goods[i].meanValue*amount;
					control.reserved[i]:-townhouse.reserved[i];
					townhouse.reserved[i]=0;
				EndIf
			Next
			finished=True;
		EndIf
		If(Not townhouse)
			finished=True;
		EndIf
	End Method
	Method setAnimation()
		If(Not invisible)
			worker.frame=TWorker.FR_PICK;
		Else
			worker.frame=-1;
		EndIf
	End Method
	Method getId:Int()
		Return BUY;
	End Method
	Method saveToFile(out:TStream)
		Super.saveToFile(out);
		WriteInt(out, control.faction);
		WriteInt(out, townhouse.coords.x);
		WriteInt(out, townhouse.coords.y);
		WriteDouble(out, duration);
	End Method
	Method loadFromFile(in:TStream)
		Super.loadFromFile(in);
		Local faction:Int=ReadInt(in);
		control=GameManager.players[faction];
		Local x:Int=ReadInt(in);
		Local y:Int=ReadInt(in);
		If(TTownHouse(GameManager.tilemap.data[x, y].structure))
			townhouse=TTownHouse(GameManager.tilemap.data[x, y].structure);
		Else
			townhouse=Null;
		EndIf
		duration=ReadDouble(in);
	End Method
End Type

Type TJob
	Field structure:THouse;
	Field amount:Int;
	Field id:Int;
	Field lifetime:Double;
	Method init:TJob(struct:THouse, amount:Int, id:Int, lt:Double=10)
		structure=struct;
		Self.amount=amount;
		Self.id=id;
		lifetime=lt;
		structure.reserved[id]:+amount;
		ListAddLast(GameManager.players[structure.faction].jobs, Self);
		Return Self;
	End Method
	Method update(dt:Double)
		lifetime:-dt;
		If(lifetime<=0)
			finished();
			ListRemove(GameManager.players[structure.faction].jobs, Self);
		EndIf
	End Method
	Method finished()
		If(structure)
			structure.reserved[id]:-amount;
		EndIf
	End Method
	Function compareFunction:Int(obj1:Object, obj2:Object)
		If(TJob(obj1) And TJob(obj2))
			If( TJob(obj1).getValue() < TJob(obj2).getValue())
				Return -1;
			Else
				Return 1;
			EndIf
		EndIf
		Return 0;
	End Function
	
	Method getValue:Double()
		If(GameManager.world.goods[id].meanValue>0)
			Return lifetime*GameManager.players[structure.faction].storage[id]/GameManager.world.goods[id].meanValue;
		Else
			Return lifetime*GameManager.players[structure.faction].storage[id];
		EndIf
	End Method
	
	Method saveToFile(out:TStream)
		WriteInt(out, structure.coords.x);
		WriteInt(out, structure.coords.y);
		WriteInt(out, amount);
		WriteInt(out, id);
		WriteDouble(out, lifetime);
	End Method
	Method loadFromFile(in:TStream)
		Local x:Int=ReadInt(in);
		Local y:Int=ReadInt(in);
		If(THouse(GameManager.tilemap.data[x, y].structure))
			structure=THouse(GameManager.tilemap.data[x, y].structure);
		Else
			structure=Null;
		EndIf
		amount=ReadInt(in);
		id=ReadInt(in);
		lifetime=ReadDouble(in);
	End Method
End Type
